/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.annotation;


/**
 * This class collects "marker methods" that can be used to mark interesting spots in an
 * algorithm. These spots are considered to be costs of some kind, e.g. key comparisons.
 * <p/>
 * <p/> <b>All invocations of these methods have to take <em>constant</em> parameters.</b>
 * For example, {@code produceCost("comparisons", 1)} is OK, but {@code
 * produceCost(x.toString(), y+2)} is <em>not</em> allowed.
 *
 * @author Sebastian Wild
 */
public final class Annotations {


   /**
    * Call sites of this method are marked as "location of interest" in the sense of cost.
    * For example, if you are interested in the number of comparisons, call this method
    * with {@code costMeasure} {@code "comparisons"} each time a comparison has been made.
    * <br /> <p/> The {@code amount} parameter allows for non-uniform costs, but the
    * passed value must be a constant <br /> <p/> Each {@code costMeasure} is counted
    * separately, in order to allow analysis of e.g. comparisons and write accesses at the
    * same time.
    *
    * @param costMeasure the name of the cost measure this annotation amounts to
    * @param amount      the amount of costs represented by this annotation
    * @return true (for annotating boolean expressions)
    */
   public static boolean produceCost(final String costMeasure, final int amount) {
      return true;
   }

   /**
    * This method is used to mark locations in an algorithm, that indicate that a certain
    * amount of input data has been consumed, i.e. a certain amount of input data has been
    * processed completely.
    *
    * @param amount the number of input size units consumed at the current location
    * @deprecated currently not used in analysis
    */
   public static boolean consumeInput(final int amount) {
      return true;
   }

   /**
    * This method can be used to collect some information while profiling that goes beyond
    * counting the transitions from one basic block to another. It can be seen as debug
    * means. <b>MaLiJAn internal analyses do not profit form trigger events.</b>
    * <p/>
    * <p/>
    * Call sites of this methods will yield an event containing the (dynamically created)
    * content. The list of events fired during profiling can then be further analysed
    * externally.
    *
    * @param type
    * @param content
    * @return
    */
   public static boolean triggerEvent(final String type, final String content) {
      return true;
   }


   public static final String PRODUCE_COST = "produceCost";
   public static final String CONSUME_INPUT = "consumeInput";
}
