/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unikl.cs.agak.malijan.input;

import de.unikl.cs.agak.malijan.input.annotations.Simple;

import java.io.Serializable;
import java.util.NoSuchElementException;

/**
 * Abstract base class for input providers that can create inputs of arbitrary sizes.
 * Subclasses only need to override {@link #createInput(long)}.
 * <p/>
 * This input provider has {@code perSize} inputs for each input size, from {@code
 * minSize} up to {@code maxSize}, in steps of {@code stepSize}.
 *
 * @author Sebastian Wild (s_wild@cs.uni-kl.de)
 */
public abstract class AbstractSizeRangeInputProvider<T extends Serializable>
	  implements InputProvider<T> {

	private final long perSize;
	private final long minSize;
	private final long maxSize;
	private final long stepSize;
	private long nextSize;
	private long ofSize = 0;


	public AbstractSizeRangeInputProvider(
		  @Simple(label = "Minimum Size", val = "100") final long minSize,
		  @Simple(label = "Maximum Size", val = "1000") final long maxSize,
		  @Simple(label = "Per Size", val = "100") final long perSize,
		  @Simple(label = "Size Increment", val = "10") final long stepSize) {
		this.perSize = perSize;
		this.minSize = minSize;
		this.maxSize = maxSize;
		this.stepSize = stepSize;

		this.nextSize = minSize;
	}

	@Override
	public boolean hasNext() {
		return maxSize < 0 || nextSize < maxSize || (nextSize == maxSize
			  && ofSize < perSize);
	}

	@Override
	public final Input<T> next() {
		if (hasNext()) {
			final long size = this.nextSize;

			ofSize++;

			if (ofSize >= perSize) {
				nextSize += stepSize;
				ofSize = 0;
			}

			return new InputImpl<T>(size, createInput(size));
		} else {
			throw new NoSuchElementException();
		}
	}

	/**
	 * To be overwritten in subclasses. Creates one new instance of input data for the given
	 * input size.
	 *
	 * @param size
	 * @return
	 */
	abstract protected T createInput(final long size);

	@Override
	public long size() {
		return maxSize >= 0 ? perSize * ((maxSize - minSize) / stepSize + 1) : UNKNOWN_SIZE;
	}


	@Override
	public abstract String toString();

	/**
	 * This method returns a String representation of the parameters. It should be used by
	 * the toString implementation of the subclass.
	 *
	 * @return the parameters separated by commata.
	 */
	protected String getParameterString() {
		return minSize + "," + maxSize + "," + perSize + "," + stepSize;
	}


}
