/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.input;

import de.unikl.cs.agak.malijan.util.ViewFormat;

import java.io.Serializable;
import java.lang.reflect.Array;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * A default implementation of {@link Input} that provides meaningful implementations of {@link #equals}, {@link
 * #hashCode} and {@link #toString}. IDs are chosen globally unique.
 *
 * @author Sebastian Wild (s_wild@cs.uni-kl.de)
 */
public final class InputImpl<T extends Serializable> implements Input<T> {

  private static final long serialVersionUID = 379981074196550563L;

  private static final AtomicInteger nextFreeId = new AtomicInteger(0);

  private final long size;

  private final T data;

  private final int id;

  public InputImpl(final long size, final T data) {
	 if (size < 0L) throw new IllegalArgumentException("Input size may not be negative.");
    this.size = size;
    this.data = data;
    id = nextFreeId.getAndIncrement();
  }

  @Override
  public long size() {
    return size;
  }

  @Override
  public T data() {
    return data;
  }

  @Override
  public long id() {
    return id;
  }

  @Override
  public boolean equals(final Object o) {
    return this == o || !(o == null || getClass() != o.getClass())
	      && id == ((InputImpl) o).id;
	}

	@Override
  public int hashCode() {
    return id;
  }

  @Override
  public String view(final ViewFormat format) {
    final StringBuilder result = new StringBuilder("#" + id + " (size " + size + "): ");
    if ( data != null && data.getClass().isArray() ) {
      result.append("[");
      final int length = Array.getLength(data);
      for ( int i = 0; i < length; i++ ) {
        result.append(Array.get(data, i));
        result.append(i < length - 1 ? ", " : "");
      }
      result.append("]");
    }
    else if (data!=null) {
      result.append(data.toString());
    } else {
	    result.append("null");
    }
    return result.toString();
  }

	@Override
	public String toString() {
		final String dataString = data == null ? "null" : data.getClass().getSimpleName();
		return "Input(" + dataString + "," + id + ")";
	}
}
