/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.input;

import java.io.Serializable;

/**
 * <p> Implement this interface to provide an analysed algorithm with input data. This
 * generic interface can be seen as a stream of {@link Input}s. </p>
 * <p/>
 * <p>
 * To use the input provider in Malijan-GUI client, it must have public constructors and
 * those constructors may only take arguments of “simple” types, namely:
 * <ul>
 *    <li>all primitive types (byte, short, int, long, boolean, char, float, double)</li>
 *    <li>{@link String}s</li>
 *    <li>{@link java.io.File}s, i.e. a filename representing one file or directory</li>
 *    <li>arrays of “simple” types</li>
 * </ul>
 * Annotate their parameters with {@link de.unikl.cs.agak.malijan.input.annotations.Simple}
 * or {@link de.unikl.cs.agak.malijan.input.annotations.Choice} to allow nice GUI
 * representations.
 * </p>
 * <p>Choice allows to restrict possible values to a finite set of alternatives. These
 * have to be given as {@link de.unikl.cs.agak.malijan.input.annotations.Choice#choices()}</p>.
 * <p/>
 * <p>
 * You can annotate class and constructors with {@link de.unikl.cs.agak.malijan.input.annotations.Label}
 * to show info messages in GUI.
 *</p>
 * @param <T> The type of underlying input values
 * @author Raphael Reitzig, 09.2011
 * @see de.unikl.cs.agak.malijan.input.annotations Annotations for UI interaction
 * @see de.unikl.cs.agak.malijan.input.defaults Default implementations
 */
public interface InputProvider<T extends Serializable> {
	/** Constant for unknown input number. */
	long UNKNOWN_SIZE = -1;

  /**
   * Checks whether there are more inputs to be retrieved.
   *
   * Several calls to this method without a call to {@link #next()} in between must return
   * the same value.
   *
   * @return {@code true} iff there is another input
   * @throws Throwable in case of any problems
   */
   boolean hasNext() throws Throwable;

  /**
   * Generates the next input.
   * @return The next input if there is one. {@code null} if there are no more inputs
   * @throws java.util.NoSuchElementException in case no more inputs are available
   */
   Input<T> next() throws Throwable;

   /**
    * @return the total number of inputs to be retrieved from this
    *         {@code InputProvider}. If this quantity can not be determined,
    *         {@link #UNKNOWN_SIZE} is returned.
    * @throws Throwable in case of any problems
    */
   long size() throws Throwable;
}
