/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.util;

import java.util.LinkedHashMap;
import java.util.Map;

public enum ViewFormat {

	ASCII(System.getProperty("line.separator"), "->", " -- ", "\t", " ",
		  "_____________________________________________" + System
				 .getProperty("line.separator")+System.getProperty("line.separator")) {
		@Override public String small(final String string) {
			return string;
		}

		@Override
		public String emph(final String text) {
			return text;
		}

		@Override
		public String strong(final String text) {
			if (text == null) return emph("NULL");
			return text.toUpperCase();
		}

		@Override
		public String unorderedList(final String... items) {
			if (items == null) return emph("NULL");
			StringBuilder result = new StringBuilder(items.length * 20);
			result.append(endl);
			for (String item : items) {
				if (item != null) {
					result.append(" -- ").append(item).append(endl);
				}
			}
			return result.toString();
		}

		@Override
		public String orderedList(final String... items) {
			if (items == null) return emph("NULL");
			final String formatString = " %0" + ceilLog10(items.length + 1) + "d. ";
			StringBuilder result = new StringBuilder(endl);
			int itemNumber = 1;
			for (String item : items) {
				if (item != null) {
					result.append(String.format(formatString, itemNumber++)).append(item)
						  .append(endl);
				}

			}
			return result.toString();
		}

		@Override
		public String heading(final String head) {
			if (head == null) return "#####   NULL   #####";
			return new StringBuilder(head.length() + 20).append("#####   ").append(head)
				  .append("   #####").append(endl).toString();
		}

		@Override
		public String littleHead(final String head) {
			return "   " + head + endl;
		}

		@Override
		public String description(final Map<?, ?> map) {
			if (map == null) return emph("NULL");
			final StringBuilder result = new StringBuilder(map.size() * 30);
			int maxLen = 0;
			for (Object s : map.keySet()) {
				maxLen = Math.max(maxLen, s.toString().length());
			}
			maxLen += 1;
			final String format = " %-" + maxLen + "s: ";
			for (final Map.Entry<?, ?> entry : map.entrySet()) {
				result.append(String.format(format, toString(entry.getKey())));
				result.append(toString(entry.getValue())).append(endl);
			}
			return result.toString();
		}

		@Override public String table(final String[][] rows) {
			if (rows == null) return emph("NULL");
			final StringBuilder result = new StringBuilder(rows.length * 30);
			int columns = 0;
			for (final String[] row : rows) {
				columns = Math.max(columns, row.length);
			}
			if (columns == 0) return "";
			final int[] maxLen = new int[columns];
			for (final String[] row : rows) {
				for (int i = 0; i < row.length; i++) {
					final String cell = row[i];
					maxLen[i] = Math.max(maxLen[i], cell.length());
				}
			}
			for (int i = 0; i < maxLen.length; i++) {
				maxLen[i] += 2; // space between columns
			}
			final String[] format = new String[columns];
			for (int i = 0; i < format.length; i++) {
				format[i] = "%-" + maxLen[i] + "s";
			}
			for (final String[] row : rows) {
				for (int i = 0; i < row.length; i++) {
					result.append(String.format(format[i], row[i]));
				}
				result.append(endl);
			}
			return result.toString();
		}

		@Override
		public String escape(final String string) {
			return string;
		}

		@Override
		public String code(final String string) {
			return string;
		}
	},
	HTML("<p/>", " ‣ ", " &mdash; ", "&nbsp;&nbsp;&nbsp;&nbsp;","&thinsp;","<hr/>") {
		@Override
		public String emph(final String text) {
			if (text == null) return emph("NULL");
			return new StringBuilder(text.length() + 9).append("<em>").append(text).append(
				  "</em>").toString();
		}

		@Override
		public String strong(final String text) {
			if (text == null) return emph("NULL");
			return new StringBuilder(text.length() + 7).append("<b>").append(text).append(
				  "</b>").toString();
		}

		@Override
		public String orderedList(final String... items) {
			if (items == null) return emph("NULL");
			StringBuilder result = new StringBuilder(items.length * 25);
			result.append("<ol>");
			for (String item : items) {
				if (item != null && !"".equals(item)) {
					result.append("<li>").append(item).append("</li>");
				}
			}
			return result.append("</ol>").toString();
		}

		@Override
		public String unorderedList(final String... items) {
			if (items == null) return emph("NULL");
			StringBuilder result = new StringBuilder(items.length * 25);
			result.append("<ul>");
			for (String item : items) {
				if (item != null && !"".equals(item)) {
					result.append("<li>").append(item).append("</li>");
				}
			}
			return result.append("</ul>").toString();
		}

		@Override
		public String heading(final String head) {
			if (head == null) return heading("NULL");
			return new StringBuilder(head.length() + 9).append("<h3>").append(head).append(
				  "</h3>").toString();
		}

		@Override
		public String littleHead(final String head) {
			if (head == null) return littleHead("NULL");
			return new StringBuilder(head.length() + 9).append("<h4>").append(head).append(
				  "</h4>").toString();
		}

		@Override
		public String description(final Map<?, ?> map) {
			if (map == null) return emph("NULL");
			final StringBuilder r = new StringBuilder(map.size() * 30);
			r.append("<table>");
			for (final Map.Entry<?, ?> entry : map.entrySet()) {
				r.append("<tr valign='baseline'><td>");
				if (entry.getKey() instanceof Viewable) {
					r.append(toString(entry.getKey())).append(":");
				} else {
					r.append(emph(entry.getKey() + ":"));
				}
				r.append("</td><td>");
				r.append(toString(entry.getValue()));
				r.append("</td></tr>");
			}
			return r.append("</table>").toString();
		}

		@Override public String table(final String[][] rows) {
			if (rows == null) return emph("NULL");
			final StringBuilder r = new StringBuilder(rows.length * 30);
			r.append("<table>");
			for (final String[] row : rows) {
				r.append("<tr valign='baseline'>");
				for (final String cell : row) {
					r.append("<td>").append(cell).append("</td>");
				}
				r.append("</tr>");
			}
			return r.append("</table>").toString();
		}

		@Override
		public String escape(final String string) {
			if (string == null) return emph("NULL");
			return string.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
		}

		@Override
		public String code(final String string) {
			if (string == null) return emph("NULL");
			final StringBuilder result = new StringBuilder(string.length() + 20);
			result.append("<code>");
			result.append(escape(string));
			result.append("</code>");
			return result.toString();
		}

		@Override public String small(final String string) {
			return "<small>" + string + "</small>";
		}
	}; //→‣

	private ViewFormat(final String endl, final String rightarrow, final String dash,
		  final String tab, final String thinspace, final String hline) {
		this.endl = endl;
		this.rightarrow = rightarrow;
		this.dash = dash;
		this.tab = tab;
		this.thinspace = thinspace;
		this.hline = hline;
	}

	/** System-dependent new line String */
	public final String endl;
	/** right arrow symbol */
	public final String rightarrow;
	public final String dash;
	public final String tab;

	public final String thinspace;

	public final String hline;

	public abstract String strong(String text);

	public abstract String emph(String text);

	/**
	 * Print formatted list
	 *
	 * @param items items, null entries will be ignored
	 * @return list as string
	 */
	public abstract String orderedList(String... items);

	/**
	 * Print formatted list
	 *
	 * @param items items, null entries will be ignored
	 * @return list as string
	 */
	public abstract String unorderedList(String... items);

	public abstract String heading(String head);

	public abstract String littleHead(String head);

	public String description(String[] keys, String[] values) {
		if (keys.length != values.length) {
			throw new IllegalArgumentException("need same number of keys and values");
		}
		Map<Object, Object> map = new LinkedHashMap<Object, Object>(
			  keys.length * 4 / 3 + 1);
		for (int i = 0; i < keys.length; i++) {
			map.put(keys[i], values[i]);
		}
		return description(map);
	}

	public abstract String table(String[][] rows);

	public abstract String description(Map<?, ?> map);

	private static int ceilLog10(final int n) {
		assert n >= 0;
		int log = 1;
		int m = 10;
		while (m < n) {
			m *= 10;
			++log;
		}
		return log;
	}

	/**
	 * Escape the given string, if it contains active elements in the used format, e.g.
	 * 'tags' {@code <someText>} in HTML
	 *
	 * @param string
	 * @return
	 */
	public abstract String escape(String string);

	/**
	 * Put the given string as escaped in HTML code environment.
	 *
	 * @param string
	 * @return
	 */
	public abstract String code(String string);

	/**
	 * Show the given string in small font.
	 * @param string
	 * @return
	 */
	public abstract String small(String string);


	public String toString(final Object o) {
		if (o instanceof Viewable) {
			final Viewable viewable = (Viewable) o;
			return viewable.view(this);
		} else if (o == null) {
			return this.emph("NULL");
		} else {
			return o.toString();
		}
	}


}
